package com.peup.dao.impl;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.support.TransactionTemplate;

import com.peup.dao.IGenericDao;

@Repository
public abstract class GenericDao<T> implements IGenericDao<T> {
	private final static Logger logger = Logger.getLogger(GenericDao.class);
	@Autowired
	@PersistenceContext
	private  EntityManager em;
	 private TransactionTemplate transactionTemplate;
	private Class<T> type;

	public GenericDao() {
		Type t = getClass().getGenericSuperclass();
		ParameterizedType pt = (ParameterizedType) t;
		type = (Class) pt.getActualTypeArguments()[0];
	}
	
	public long countAll(final Map<String, Object> params) {

		final StringBuffer queryString = new StringBuffer(
				"SELECT count(o) from ");

		queryString.append(type.getSimpleName()).append(" o ");
		queryString.append(this.getQueryClauses(params, null));

		final Query query = this.em.createQuery(queryString.toString());

		return (Long) query.getSingleResult();

	}
	public T create(final T t) {
		this.em.persist(t);
		return t;
	}
	public void delete(final Object id) {
		this.em.remove(this.em.getReference(type, id));
	}

	public T find(final Object id) {
		return (T) this.em.find(type, id);
	}

	public List<T> find(final Map<String, Object> params) {

		final StringBuffer queryString = new StringBuffer("SELECT o from ");

		queryString.append(type.getSimpleName()).append(" o ");
		queryString.append(this.getQueryClauses(params, null));

		final Query query = this.em.createQuery(queryString.toString());

		return  query.getResultList();

	}
	public T update(final T t) {
		 
		 T inter=this.em.merge(t);		
		return inter;
	}

	/*
	 * @Override public List<GenericEntity> getByID(Long id) {
	 * 
	 * String query = "from "+GenericEntity.class.getName()
	 * +" where upper(codeCategorie) = '" + code.toUpperCase() + "'"; if
	 * (idCategorie != 0) query += " and id <> " + idCategorie; return
	 * (Categorie) getSessionFactory().getCurrentSession()
	 * .createQuery(query).uniqueResult(); }
	 * 
	 * 
	 * public List<?>getAll() { StringBuilder sb=new StringBuilder();
	 * sb.append("from "+GenericEntity.class.getName());
	 * //query.setParameter("codePostal", codePostal);
	 * //query.setParameter("organismeId", organismeId);
	 * 
	 * Query jpqlQuery = getEntityManager().createQuery(sb.toString());
	 * 
	 * return jpqlQuery.getResultList();
	 * 
	 * }
	 */

	private String getQueryClauses(final Map<String, Object> params,
			final Map<String, Object> orderParams) {
		final StringBuffer queryString = new StringBuffer();
		if ((params != null) && !params.isEmpty()) {
			queryString.append(" where ");
			for (final Iterator<Map.Entry<String, Object>> it = params
					.entrySet().iterator(); it.hasNext();) {
				final Map.Entry<String, Object> entry = it.next();
				if (entry.getValue() instanceof Boolean) {
					queryString.append(entry.getKey()).append(" is ")
							.append(entry.getValue()).append(" ");
				} else {
					if (entry.getValue() instanceof Number) {
						queryString.append(entry.getKey()).append(" = ")
								.append(entry.getValue());
					} else {
						// string equality
						queryString.append(entry.getKey()).append(" = '")
								.append(entry.getValue()).append("'");
					}
				}
				if (it.hasNext()) {
					queryString.append(" and ");
				}
			}
		}
		if ((orderParams != null) && !orderParams.isEmpty()) {
			queryString.append(" order by ");
			for (final Iterator<Map.Entry<String, Object>> it = orderParams
					.entrySet().iterator(); it.hasNext();) {
				final Map.Entry<String, Object> entry = it.next();
				queryString.append(entry.getKey()).append(" ");
				if (entry.getValue() != null) {
					queryString.append(entry.getValue());
				}
				if (it.hasNext()) {
					queryString.append(", ");
				}
			}
		}
		return queryString.toString();
	}
			
	public TransactionTemplate getTransactionTemplate() {
        return this.transactionTemplate;
    }
 
    public void setTransactionTemplate(TransactionTemplate transactionTemplate) {
        this.transactionTemplate = transactionTemplate;
    }
}